import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Properties;

public class TestMacroProcessor {
	private static final String MACRO_PREFIX = "//#";
	
	private File macroFolder = new File("macros");

	public static void main(String[] args) throws IOException {
		File configFile = new File("config.properties");
		if (!configFile.exists()) {
			throw new FileNotFoundException("No config.properties file found in current path");
		}
		
		Properties config = new Properties();
		config.load(new FileInputStream(configFile));
		
		TestMacroProcessor testMacroProcessor = new TestMacroProcessor();
		testMacroProcessor.macroFolder = new File(config.getProperty("macro.path"));
		testMacroProcessor.run(new File(config.getProperty("input.ats.path")), new File(config.getProperty("output.ats.path")));
	}

	public void run(File inputPath, File outputPath) throws IOException {
		ArrayList<File> files = new ArrayList<File>();
		visitAllFiles(inputPath, files, ".ttcn_");
		
		for (File inputFile : files) {
			runForFile(inputFile, inputPath, outputPath);
		}
	}

	public void runForFile(File inputFile, File inputPath, File outputPath) throws IOException {
		File targetFile = computeTargetFileName(inputFile, inputPath, outputPath);
		targetFile.getParentFile().mkdirs();
		replaceInFile(inputFile, targetFile);
	}

	/**
	 * Expecting //# TC or //# TC2(param1, param2)
	 */
	private void replaceInFile(File inputFile, File outputFile) throws IOException {
		BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(inputFile), "UTF-8"));
		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(outputFile), "UTF-8"));
		try {
			String line;
			while ((line = br.readLine()) != null) {
				if (line.trim().startsWith(MACRO_PREFIX)) {
					int indexOfMacroPrefix = line.indexOf(MACRO_PREFIX);
					int beginIndex = indexOfMacroPrefix+MACRO_PREFIX.length();
					int endIndex = line.indexOf('(', beginIndex);
					if (endIndex <= beginIndex) {
						endIndex = line.length();
					}
					String macro = line.substring(beginIndex, endIndex).trim();
					bw.newLine();
					String moduleName = inputFile.getName();
					moduleName = moduleName.substring(0, moduleName.lastIndexOf('.'));
					writeMacro(macro, bw, moduleName);
				} else {
					bw.write(line);
					bw.newLine();
				}
			}
		} finally {
			try {
				br.close();
			} catch (IOException e) {
				// ignore
			}
			try {
				bw.close();
			} catch (IOException e) {
				// ignore
			}

		}
	}
	
	private String readMacro(BufferedReader br) throws IOException {
		StringBuilder sb = new StringBuilder();
		String line;
		while ((line = br.readLine()) != null) {
			sb.append(line).append("\n");
		}
		return sb.toString();
	}

	private void writeMacro(String macro, BufferedWriter bw, String moduleName) throws IOException {
		File macroFile = new File(macroFolder, macro+".ttcn_macro");
		BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(macroFile), "UTF-8"));
		String macroString = readMacro(br);
		macroString = macroString.replace("${module}", moduleName);
		bw.write(macroString);
		bw.newLine();
		try {
			br.close();
		} catch (Exception e) {
			// ignore
		}
	}

	private File computeTargetFileName(File inputFile, File inputPath, File outputPath) throws IOException, IOException {
		String canonicalInputFile = inputFile.getCanonicalPath();
		String canonicalInputPath = inputPath.getCanonicalPath();
		String relativeInputPath = canonicalInputFile.substring(canonicalInputPath.length()+1);
		if (!new File(inputPath, relativeInputPath).equals(inputFile)) {
			throw new IOException("parent inputPath not matching inputFile");
		}
		// cut _ at end of file
		if (relativeInputPath.endsWith(".ttcn_")) {
			relativeInputPath = relativeInputPath.substring(0, relativeInputPath.length()-1);
		}
		return new File(outputPath, relativeInputPath);
	}

	public static void visitAllFiles(File f, ArrayList<File> files, String extension) {
		if (f.isDirectory()) {
			String[] children = f.list();
			for (int i = 0; i < children.length; i++) {
				visitAllFiles(new File(f, children[i]), files, extension);
			}
		} else {
			if (f.toString().endsWith(extension)) {
				files.add(f);
			}
		}
	}

}
